package mv3302;

import mv3302.sensor.SimpleCookieCutterSensor;
import simkit.SimEntityBase;
import simkit.smd.Mover;

/**
 * The SimpleCookieCutterMediator class represents a mediator that interacts
 * with a SimpleCookieCutterSensor to handle the detection and undetection of a
 * Mover object. It extends the SimEntityBase class, which is a base class for
 * simulation entities in SimKit.
 */
public class SimpleCookieCutterMediator extends SimEntityBase {

    /**
     * Handles the event when a Mover enters the range of the
     * SimpleCookieCutterSensor. If the Mover is not already in the sensor's
     * contacts, it waits for a "Detection" event to occur.
     *
     * @param target The Mover object that entered the sensor's range.
     * @param sensor The SimpleCookieCutterSensor responsible for detection.
     */
    public void doEnterRange(Mover target, SimpleCookieCutterSensor sensor) {
        if (!sensor.getContacts().contains(target)) {
            sensor.waitDelay("Detection", 0.0, target);
        }
    }

    /**
     * Handles the event when a Mover exits the range of the
     * SimpleCookieCutterSensor. If the Mover is not already in the sensor's
     * contacts, it waits for an "Undetection" event to occur. If the Mover is
     * still not detected, it interrupts the ongoing "Detection" event.
     *
     * @param target The Mover object that exited the sensor's range.
     * @param sensor The SimpleCookieCutterSensor responsible for detection.
     */
    public void doExitRange(Mover target, SimpleCookieCutterSensor sensor) {
        if (!sensor.getContacts().contains(target)) {
            sensor.waitDelay("Undetection", 0.0, target);
        }
        if (!sensor.getContacts().contains(target)) {
            interrupt("Detection", 0.0, target);
        }
    }
}
