
package mv3302.run;

import java.awt.geom.Point2D;
import java.util.Arrays;
import mv3302.SimpleConstantRateMediator;
import mv3302.SimpleMover;
import mv3302.SimplePathMoverManager;
import mv3302.SimpleReferee;
import mv3302.sensor.SimpleConstantRateSensor;
import simkit.Schedule;
import simkit.random.RandomVariate;
import simkit.random.RandomVariateFactory;
import simkit.smd.Mover;
import simkit.smd.Sensor;
import simkit.util.SimplePropertyDumper;

/**
 *
 * @author dansl
 */
public class Assignment7ConstantRate1Mover {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        RandomVariate exponentialGenerator = RandomVariateFactory.getInstance("Exponential", .7);
        Mover[] targets = new Mover[]{
            new SimpleMover(new Point2D.Double(30.0, 40.0), 20.0),
            new SimpleMover(new Point2D.Double(0, 0), 20.0)
        };

        Point2D[] targetPath = new Point2D[]{new Point2D.Double(-30.0, -40.0), new Point2D.Double(60.0, 80.0)};
        Point2D[] sensorPath = new Point2D[]{new Point2D.Double(30.0, 40.0)};
        SimplePathMoverManager[] targetMoverManager = new SimplePathMoverManager[]{
            new SimplePathMoverManager(targetPath, true),
            new SimplePathMoverManager(sensorPath, true)
        };

        Sensor[] sensors = new Sensor[targets.length];
        double meanTimeToDetect = 5.0;
        sensors[0] = new SimpleConstantRateSensor(targets[0], 30, meanTimeToDetect);
        sensors[1] = new SimpleConstantRateSensor(targets[1], 40, meanTimeToDetect);

        SimplePropertyDumper simplePropertyDumper = new SimplePropertyDumper(true);
        for (Sensor sensor : sensors) {
            sensor.addPropertyChangeListener(simplePropertyDumper);
        }

        for (int i = 0; i < targets.length; ++i) {
            targets[i].addSimEventListener(targetMoverManager[i]);
            targetMoverManager[i].addSimEventListener(targets[i]);
        }

        SimpleReferee simpleReferee = new SimpleReferee(Arrays.asList(sensors),
                Arrays.asList(targets));
        System.out.println(simpleReferee);

        SimpleConstantRateMediator simpleConstantRateMediator = new SimpleConstantRateMediator(exponentialGenerator);
        simpleReferee.addSimEventListener(simpleConstantRateMediator);

        Schedule.setVerbose(true);
        Schedule.reset();
        Schedule.startSimulation();
    }

}
